/*******************************************
 *
 * Wohlfhlmaschine II
 *
 * (C) Fingers elektrische Welt 2005
 *
 * Prozessor : PIC16F76
 *
 *******************************************/

#include	<pic.h>
#include	<stdlib.h>
#include  <stdio.h>
#include	<string.h>
#include  <math.h>

__CONFIG(0x3f32);

//---------------------------------------
//  Configuration word:
//	Bit 13,12	Code protection		11	=OFF
//	Bit 11,10	Code protection		11	=OFF
//	Bit 9,8		Code protection		11	=OFF
//	Bit 7		not implemented		x
//	Bit 6		brown out			0	=disabled
//	Bit 5,4		Code protection		11	=OFF
//	Bit	3		Power up timer		0	=enabled
//	Bit 2		Watchdog			0	= disabled
//	Bit 1,0		Oszillator			10	=HS
//----------------------------------------

#define FALSE          0
#define TRUE           1
#define StartPause    20
#define EndPause       1

#define Timeout       5000

char Motorleistung = 0;
char TastFlag = FALSE;
long Timer;

//---------------------
// Watchdog rcksetzen
//---------------------
void WDR (void)
 {
  #asm
  clrwdt
  #endasm
 }

//------------------------------
// Einfache Verzgerungsroutine
//------------------------------
void Delay (unsigned int time)
 {
   unsigned char ttt;

   while (time > 0)
   	{
   	 ttt = 127;
   	 while (ttt>0) { ttt--; }
   	 WDR ();
   	 time-- ;
   	}

 }

/*******************
 *
 * Einfaches Delay
 *
 *******************/
void Delayms (unsigned int wert)
 {
  unsigned int i;

  for (i=0; i< wert; i++) Delay (2);
 }

/*************************
 *
 * Zentrale ISR
 *
 *************************/
void interrupt ISR (void)
 {

  return;
 }

/*************************
 *
 * Schreibt PWM-Wert
 *
 *************************/
void SetPWM (int wert)
 {
   int puffer;

   puffer = wert;
   puffer &= 0b0000001111111100;
   puffer = (puffer >> 2);
   CCPR1L = (char)(puffer);               // Obere 8 Bits beschreiben
   CCP1CON &= 0b00001111;                 // Untere Bit auf Null
 }

/*************************
 *
 * 0-100% Motorleistung
 *
 *************************/
void SetMotor (char prozent)
 {
  SetPWM (prozent * 10);
 }

//-----------------------------
// Initialisierung des Systems
//-----------------------------
void init (void)
 {

   // Portpins klarmchen
   TRISA = 0b11111111;    // 1 = Eingang, 0 = Ausgang
   TRISB = 0b00111100;
   TRISC = 0b11111011;

   PORTC &= ~4;           // Motor aus

   // PortA als IO
   ADCON0 = 0;
   ADCON1 = 7;

   // PWM startklar machen
   PR2 =0xFF;            // 19 Khz PWM
   SetPWM (0);           // Ausgang 0
   T2CON = 0b00000000;   // Timer 2 Prescaler = 1:1
   TMR2ON = 1;           // Timer 2 einschalten
   CCP1CON = 0b00001100; // PWM einschalten

 }

/*********************************
 *
 * Fragt den vorderen Schalter ab
 *
 *********************************/
char Vorne (void)
 {
  return ((PORTA & 4) == 0);
 }

/*********************************
 *
 * Fragt den hinteren Schalter ab
 *
 *********************************/
char Hinten (void)
 {
  return ((PORTA & 2) == 0);
 }

/*********************************
 *
 * Fragt den Taster ab
 *
 *********************************/
char Taster (void)
 {
  return ((PORTA & 1) == 0);
 }

/*********************************
 *
 * Schaltet die rote Warnleuchte
 *
 *********************************/
void Rot (char art)
 {
  if (art) PORTB |=1; else PORTB &= ~1;
 }

/*********************************
 *
 * Schaltet die Richtung um
 *
 *********************************/
void Richtung (char art)
 {
  if (art) PORTB |=2; else PORTB &= ~2;
 }

/*********************************
 *
 * Not-Aus
 *
 *********************************/
void Blinker (void)
 {
  char ever = TRUE;

   PORTC &= ~4;
   SetMotor (0);

   while (ever)
    {
     Rot (TRUE);
     Delayms (300);
     Rot (FALSE);
     Delayms (300);
    }
 }

/*************************
 *
 * Softstart
 *
 *************************/
void StartRampe (char Richt)
 {
  char i=60;

  Richtung (Richt);
  Delayms (200);

  while (i < 100)
   {
    SetMotor (i);
    Delayms (StartPause);
    i++;
    if (Taster ()) TastFlag = TRUE;
    if ((Richt) && (Vorne ())) return;
    if ((!Richt) && (Hinten ())) return;
   }

 }

/*************************
 *
 * Softstart
 *
 *************************/
void BremsRampe (char Richt)
 {
  char i=90;

  Richtung (Richt);

  while (i > 60)
   {
    if (Taster ()) TastFlag = TRUE;
    SetMotor (i);
    Delayms (EndPause);
    i -= 5;
   }

  SetMotor (0);
  Delayms (20);
 }

/*****************
 *
 * Hauptprogramm
 *
 *****************/
void main (void)
 {
  init ();                                   // Alles startklar machen

  while (1)
   {

    // Vorwrtslauf bis zum vorderen Schalter
    StartRampe (TRUE);
    Timer = 0;
    while (!Vorne ())
     {
      if (Taster ()) TastFlag = TRUE;
      Timer++;
      Delayms (1);
      if (Timer > Timeout) Blinker ();
     }

    BremsRampe (TRUE);

    // Parkposition anfahren ?
    if (TastFlag)
     {
      StartRampe (FALSE);
      Delayms (800);
      BremsRampe (FALSE);
      while (!Taster ());
      Delayms (200);
      while (Taster ());
      Delayms (200);
      TastFlag = FALSE;
     }

    // Und wieder zurck
    StartRampe (FALSE);
    Timer=0;
    while (!Hinten ())
     {
      if (Taster ()) TastFlag = TRUE;
      Timer++;
      Delayms (1);
      if (Timer > Timeout) Blinker ();
     }

    BremsRampe (FALSE);

   }

 }

